/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.intelliform;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.springframework.stereotype.Component;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;

@Component
class FormDataIncomingFileMapper {

	int countAttachments(Document document) {
		return document.getElementsByTagName("file").getLength();
	}

	List<IncomingFileGroup> mapAttachments(Document document) {
		List<IncomingFileGroup> incomingFileGroups = new ArrayList<>();
		NodeList files = document.getElementsByTagName("file");

		for (int idx = 0; idx < files.getLength(); idx++) {
			String parentNodeName = buildParentNodeName(files.item(idx).getParentNode().getNodeName());
			IncomingFileGroup group = getIncomingFileGroup(incomingFileGroups, parentNodeName);
			var updatedGroup = group.toBuilder().file(mapNodeToIncomingFile(files.item(idx))).build();
			incomingFileGroups.remove(group);
			incomingFileGroups.add(updatedGroup);
		}

		return incomingFileGroups;
	}

	String buildParentNodeName(String parentNodeName) {
		return parentNodeName.replaceAll("-item$", "");
	}

	private IncomingFile mapNodeToIncomingFile(Node item) {
		return IncomingFile.builder()
				.id(UUID.randomUUID().toString())
				.vendorId(item.getAttributes().getNamedItem("id").getNodeValue())
				.name(item.getFirstChild().getTextContent())
				.contentType(item.getAttributes().getNamedItem("content-type").getNodeValue())
				.size(Integer.valueOf(item.getAttributes().getNamedItem("length").getNodeValue()))
				.build();
	}

	private IncomingFileGroup getIncomingFileGroup(List<IncomingFileGroup> incomingFileGroups, String parentNodeName) {
		return incomingFileGroups.stream()
				.filter(group -> group.getName().equals(parentNodeName))
				.findFirst()
				.orElseGet(() -> {
					IncomingFileGroup fileGroup = IncomingFileGroup.builder().name(parentNodeName).build();
					incomingFileGroups.add(fileGroup);
					return fileGroup;
				});
	}
}
