/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.formbased;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;
import de.ozgcloud.eingang.semantik.formbased.FormBasedMapper;
import de.ozgcloud.eingang.semantik.formbased.FormBasedSemantikAdapter;

class FormBasedSemantikAdapterTest {

	@Spy
	@InjectMocks
	private FormBasedSemantikAdapter adapter;
	@Spy
	private List<FormBasedMapper> mappers = new ArrayList<>();
	@Mock
	private FormBasedMapper mapper;

	@Nested
	class TestParseFromData {

		private FormData formData = FormDataTestFactory.create();

		@BeforeEach
		void mockEngineBasedMapper() {
			when(mapper.isResponsible(any())).thenReturn(true);
			mappers.add(mapper);
		}

		@Test
		void shouldCallMapperIfResponsible() {
			adapter.parseFormData(formData);

			verify(mapper).parseFormData(formData);
		}

		@Test
		void shouldNotCallMappersIfNOTREsponsible() {
			when(mapper.isResponsible(any())).thenReturn(false);

			adapter.parseFormData(formData);

			verifyNoMoreInteractions(mapper);
		}

		@Test
		void shouldReturnMappingResult() {
			var expected = FormDataTestFactory.create();
			when(mapper.parseFormData(any())).thenReturn(expected);

			var result = adapter.parseFormData(formData);

			assertThat(result).isSameAs(expected);
		}
	}
}