package de.ozgcloud.eingang.xta;

import java.util.Objects;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.formdata.IncomingFile;

@Mapper
interface XtaMessageMapper {

	@Mapping(target = "antragsteller", ignore = true)
	@Mapping(target = "attachment", ignore = true)
	@Mapping(target = "attachments", ignore = true)
	@Mapping(target = "formData", ignore = true)
	@Mapping(target = "id", ignore = true)
	@Mapping(target = "numberOfAttachments", ignore = true)
	@Mapping(target = "numberOfRepresentations", constant = "1")
	@Mapping(target = "representation", ignore = true)
	@Mapping(target = "representations", source = "messageFiles")
	@Mapping(target = "zustaendigeStelle", ignore = true)
	@Mapping(target = "header", source = "metaData")
	FormData toFormData(XtaMessage message);

	@Mapping(target = "formEngineName", ignore = true)
	@Mapping(target = "formId", source = "messageType")
	@Mapping(target = "requestId", source = "messageId")
	@Mapping(target = "serviceKonto", ignore = true)
	@Mapping(target = "createdAt", source = "origin")
	@Mapping(target = "sender", constant = "XTA")
	@Mapping(target = "formName", constant = "dFördermittelantrag")
	FormHeader formHeaderFromMetaData(XtaMessageMetaData metaData);

	default IncomingFile toIncomingFile(XtaFile messageFile) {
		if (Objects.nonNull(messageFile)) {
			return IncomingFile.builder()
					.name(messageFile.name())
					.contentType("application/zip")
					.file(messageFile.file())
					.size(messageFile.file().length())
					.build();
		}
		return null;
	}

	default String fromId(XtaMessageId id) {
		return id.toString();
	}
}
